local position0 = require("src.frame.position")
local Position = position0.Position
local FixedPosition = position0.FixedPosition
local LayoutPosition = position0.LayoutPosition
local XAlign = position0.XAlign
local YAlign = position0.YAlign
local size1 = require("src.frame.size")
local Size = size1.Size
local window2 = require("src.frame.window")
local Window = window2.Window
local layout3 = require("src.frame.layout")
local Layout = layout3.Layout
local ColumnLayout = layout3.ColumnLayout
local RowLayout = layout3.RowLayout
local NoChildLayout = layout3.NoChildLayout
local GridLayout = layout3.GridLayout
local frame4 = require("src.frame.frame")
local Frame = frame4.Frame
local PaintFunction = frame4.PaintFunction
local rectangle5 = require("src.painter.rectangle")
local RectanglePainter = rectangle5.RectanglePainter
local image6 = require("src.painter.image")
local ImagePainter = image6.ImagePainter
local tile7 = require("src.painter.tile")
local TilePainter = tile7.TilePainter
local TileSheet = tile7.TileSheet
local Tile = tile7.Tile
local TileSetPainter = tile7.TileSetPainter
local label8 = require("src.painter.label")
local LabelPainter = label8.LabelPainter
local group9 = require("src.painter.group")
local GroupPainter = group9.GroupPainter
local ninepatch10 = require("src.painter.ninepatch")
local NinePatchPainter = ninepatch10.NinePatchPainter
local color11 = require("src.painter.color")
local Colors = color11.Colors
local Color = color11.Color
local button12 = require("src.ui.button")
local Button = button12.Button
local ActionFunction = button12.ActionFunction
local GroupButton = button12.GroupButton
local json = require("vendor.json");
local root = nil;
local toolSize = 35;
local toolTile = Tile.new(true,0,0);
local toolGroup = nil;
local selectedTile = nil;
local gameTiles = {};
local view = nil;
local resolution = 5;
local w = 4*resolution;
local h = 3*resolution;
local spacing = 1;
love.update = function(dt)
    root:update();
end
;
love.draw = function()
    love.graphics.clear(1,1,1);
    root:draw();
end
;
love.mousepressed = function(x,y,button,isTouch)
    root:press(x,y);
end
;
love.mousereleased = function(x,y,button,isTouch)
    root:release();
end
;
function IconButton(parent,size,text,tile,tilesheet,action)
    return Button.new(true,text,LayoutPosition.new(true),Size.new(true,size,size),parent,GroupPainter({RectanglePainter(Color.new(true,0.4,0.4,0.4),7),TilePainter(tilesheet,tile,Color.new(true,0.3,0.3,0.3))}),GroupPainter({RectanglePainter(Color.new(true,0.4,0.4,0.4),7),TilePainter(tilesheet,tile,Color.new(true,1,1,1))}),GroupPainter({RectanglePainter(Color.new(true,0.6,0.6,0.6),7),TilePainter(tilesheet,tile,Color.new(true,0.8,0.8,0.8))}),action)
end
love.load = function()
    local icons = TileSheet.new(true,love.graphics.newImage("assets/icons-38x39-white.png"),38,39);
    local tilemap = TileSheet.new(true,love.graphics.newImage("assets/colored_tilemap_packed.png"),14,10);
    tilemap.img:setFilter("nearest","nearest");
    root = Frame.new(true,FixedPosition.new(true,0,0),Size.new(true,800,600),Window.new(true,({ love.window.getMode() })[(0)+1],({ love.window.getMode() })[(1)+1]),NoChildLayout.new(true));
    buildToolBar(root,icons,tilemap);
    buildTileSelector(root,icons,tilemap);
    buildView(root,tilemap);
end
;
function buildToolBar(parent,icons,tilemap)
    local toolbar = Frame.new(true,FixedPosition.new(true,0,0),Size.new(true,800,toolSize),root,RowLayout.new(true,5),RectanglePainter(Colors.Gray));
    selectedTile = Frame.new(true,LayoutPosition.new(true),Size.new(true,toolSize-2,toolSize-2),toolbar,NoChildLayout.new(true),TilePainter(tilemap,toolTile));
    toolGroup = GroupButton.new(true,{IconButton(toolbar,toolSize-2,"paint-brush",Tile.new(true,21,23),icons,function(id)
        toolGroup:select(id);
    end
),IconButton(toolbar,toolSize-2,"eraser",Tile.new(true,5,11),icons,function(id)
        toolGroup:select(id);
    end
),IconButton(toolbar,toolSize-2,"eye-dropper",Tile.new(true,22,11),icons,function(id)
        toolGroup:select(id);
    end
)});
    IconButton(toolbar,toolSize-2,"save",Tile.new(true,28,27),icons,function()
        love.system.setClipboardText(json:encode(gameTiles));
    end
);
    IconButton(toolbar,toolSize-2,"download",Tile.new(true,7,10),icons,function()
        gameTiles = json:decode(love.system.getClipboardText());
        view.paint = TileSetPainter(tilemap,gameTiles,w,spacing);
    end
);
    IconButton(toolbar,toolSize-2,"ellipsis-h",Tile.new(true,32,10),icons,function()
    end
);
    toolGroup:select("paint-brush");
end
function buildTileSelector(parent,icons,tilemap)
    local tileSelector = Frame.new(true,FixedPosition.new(true,0,toolSize),Size.new(true,toolSize,600-toolSize),parent,ColumnLayout.new(true,5),RectanglePainter(Colors.Gray));
    local tiles = {};
    local updateTiles; updateTiles = function(st,mx)
        tiles = {};
        local i = 0
        while(i<mx) do
            do
                local tileX = (i+st)%tilemap.column;
                local tileY = math.floor((i+st)/tilemap.column);
                table.insert(tiles,Tile.new(true,tileX,tileY));
            end
            ::__continue0::
            i = (i+1)
        end
    end
;
    local startTileId = 0;
    local maxTiles = 13;
    updateTiles(startTileId,maxTiles);
    local spacing = 1;
    local tilesView = nil;
    IconButton(tileSelector,toolSize-2,"arrow-up",Tile.new(true,22,1),icons,function()
        if startTileId>0 then
            startTileId = (startTileId-maxTiles);
            updateTiles(startTileId,maxTiles);
            tilesView.paint = TileSetPainter(tilemap,tiles,1,spacing);
        end
    end
);
    tilesView = Frame.new(true,LayoutPosition.new(true),Size.new(true,toolSize,600-(3*(toolSize+3))),tileSelector,NoChildLayout.new(true),TileSetPainter(tilemap,tiles,1,spacing),function(self,x,y)
        local real_h = (600-(3*(toolSize+3)))/maxTiles;
        local id = startTileId+math.floor((y-self.globalPosition.y)/real_h);
        local tileX = (id)%tilemap.column;
        local tileY = math.floor((id)/tilemap.column);
        toolTile.x = tileX;
        toolTile.y = tileY;
        selectedTile.paint = TilePainter(tilemap,toolTile);
    end
);
    IconButton(tileSelector,toolSize-2,"arrow-down",Tile.new(true,19,1),icons,function()
        if startTileId<(tilemap.column*tilemap.row) then
            startTileId = (startTileId+maxTiles);
            updateTiles(startTileId,maxTiles);
            tilesView.paint = TileSetPainter(tilemap,tiles,1,spacing);
        end
    end
);
end
function buildView(parent,tilemap)
    local i = 0
    while(i<w) do
        do
            local j = 0
            while(j<h) do
                do
                    table.insert(gameTiles,Tile.new(true,1,1));
                end
                ::__continue2::
                j = (j+1)
            end
        end
        ::__continue1::
        i = (i+1)
    end
    view = Frame.new(true,FixedPosition.new(true,toolSize,toolSize),Size.new(true,800-toolSize,600-toolSize),parent,NoChildLayout.new(true),TileSetPainter(tilemap,gameTiles,w,spacing),function(self,x,y)
        local real_w = (800-toolSize)/w;
        local real_h = (600-toolSize)/h;
        local tileX = math.floor((x-self.globalPosition.x)/real_w);
        local tileY = math.floor((y-self.globalPosition.y)/real_h);
        local id = tileX+(tileY*w);
        if toolGroup.selected=="eraser" then
            gameTiles[(id)+1].x = 1;
            gameTiles[(id)+1].y = 1;
            self.paint = TileSetPainter(tilemap,gameTiles,w,spacing);
        elseif toolGroup.selected=="eye-dropper" then
            toolTile.x = gameTiles[(id)+1].x;
            toolTile.y = gameTiles[(id)+1].y;
            selectedTile.paint = TilePainter(tilemap,toolTile);
            toolGroup:select("paint-brush");
        elseif toolGroup.selected=="paint-brush" then
            gameTiles[(id)+1].x = toolTile.x;
            gameTiles[(id)+1].y = toolTile.y;
            self.paint = TileSetPainter(tilemap,gameTiles,w,spacing);
        end
    end
);
end
