local exports = exports or {}
local position0 = require("src.frame.position")
local Position = position0.Position
local FixedPosition = position0.FixedPosition
local XAlign = position0.XAlign
local YAlign = position0.YAlign
local size1 = require("src.frame.size")
local Size = size1.Size
local window2 = require("src.frame.window")
local Window = window2.Window
local layout3 = require("src.frame.layout")
local Layout = layout3.Layout
local NoChildLayout = layout3.NoChildLayout
local Frame = Frame or {}
Frame.__index = Frame
function Frame.new(construct, ...)
    local self = setmetatable({}, Frame)
    self.kind = "frame"
    if construct and Frame.constructor then Frame.constructor(self, ...) end
    return self
end
function Frame.constructor(self,pos,size,parent,layout,paint,onPress,onRelease)
    if layout==nil then layout=NoChildLayout.new(true) end
    if paint==nil then paint=nil end
    if onPress==nil then onPress=nil end
    if onRelease==nil then onRelease=nil end
    self.currentPosition = FixedPosition.new(true);
    self.globalPosition = FixedPosition.new(true);
    self.children = {};
    self.pos = pos;
    self.size = size;
    self.parent = parent;
    if self.parent.kind=="frame" then
        table.insert(self.parent.children,self);
    end
    self.layout = layout;
    self.paint = paint;
    self.onPress = onPress;
    self.onRelease = onRelease;
    self.focused = nil;
    self.needUpdate = true;
end
function Frame.moveAt(self,x,y)
    if self.pos.kind~="fixed" then
        return
    end
    self.pos.x = x;
    self.pos.y = y;
    self.needUpdate = true;
end
function Frame.anchorAt(self,x,y)
    if self.pos.kind~="anchored" then
        return
    end
    self.pos:setX(x);
    self.pos:setY(y);
    self.needUpdate = true;
end
function Frame.switchPositionType(self,pos)
    self.pos = pos;
    self.needUpdate = true;
end
function Frame.resize(self,w,h)
    self.size.w = w;
    self.size.h = h;
    self.needUpdate = true;
end
function Frame.update(self,forced)
    if forced==nil then forced=false end
    if (self.needUpdate==false) and (forced==false) then
        return
    end
    if self.pos.kind=="fixed" then
        self.currentPosition.x = self.pos.x;
        self.currentPosition.y = self.pos.y;
    elseif self.pos.kind=="anchored" then
        self.currentPosition.x = (self.pos.x*(self.parent.size.w-self.size.w));
        self.currentPosition.y = (self.pos.y*(self.parent.size.h-self.size.h));
    end
    self.globalPosition.x = (self.currentPosition.x+self.parent.globalPosition.x);
    self.globalPosition.y = (self.currentPosition.y+self.parent.globalPosition.y);
    self:layoutChild();
    self.needUpdate = false;
    local i = 0
    while(i<#self.children) do
        do
            self.children[(i)+1]:update(true);
        end
        ::__continue0::
        i = (i+1)
    end
end
function Frame.layoutChild(self)
    if self.layout.kind=="NoChildLayout" then
        return
    end
    local layoutChild = {};
    local i = 0
    while(i<#self.children) do
        do
            if self.children[(i)+1].pos.kind=="layout" then
                table.insert(layoutChild,self.children[(i)+1]);
            end
        end
        ::__continue1::
        i = (i+1)
    end
    if self.layout.kind=="ColumnLayout" then
        local y = 0;
        local i = 0
        while(i<#layoutChild) do
            do
                layoutChild[(i)+1].currentPosition.x = 0;
                layoutChild[(i)+1].currentPosition.y = y;
                y = ((y+layoutChild[(i)+1].size.h)+self.layout.spacing);
            end
            ::__continue2::
            i = (i+1)
        end
    elseif self.layout.kind=="RowLayout" then
        local x = 0;
        local i = 0
        while(i<#layoutChild) do
            do
                layoutChild[(i)+1].currentPosition.x = x;
                layoutChild[(i)+1].currentPosition.y = 0;
                x = ((x+layoutChild[(i)+1].size.w)+self.layout.spacing);
            end
            ::__continue3::
            i = (i+1)
        end
    elseif self.layout.kind=="GridLayout" then
        local x = 0;
        local y = 0;
        local i = 0
        while(i<#layoutChild) do
            do
                if ((i%self.layout.col)==0) and (i~=0) then
                    y = ((y+layoutChild[(i)+1].size.h)+self.layout.spacing);
                    x = 0;
                end
                layoutChild[(i)+1].currentPosition.x = x;
                layoutChild[(i)+1].currentPosition.y = y;
                x = ((x+layoutChild[(i)+1].size.w)+self.layout.spacing);
            end
            ::__continue4::
            i = (i+1)
        end
    end
end
function Frame.draw(self)
    if self.paint then
        self.paint(self.globalPosition.x,self.globalPosition.y,self.size.w,self.size.h);
    end
    local i = 0
    while(i<#self.children) do
        do
            self.children[(i)+1]:draw();
        end
        ::__continue5::
        i = (i+1)
    end
end
function Frame.hitTest(self,x,y)
    return (((x>=self.globalPosition.x) and (x<=(self.globalPosition.x+self.size.w))) and (y>=self.globalPosition.y)) and (y<=(self.globalPosition.y+self.size.h))
end
function Frame.pick(self,x,y)
    if self:hitTest(x,y)==false then
        return nil
    end
    local found = self;
    local i = 0
    while(i<#self.children) do
        do
            local picked = self.children[(i)+1]:pick(x,y);
            if picked~=nil then
                found = picked;
            end
        end
        ::__continue6::
        i = (i+1)
    end
    return found
end
function Frame.press(self,x,y)
    self:release();
    self.focused = self:pick(x,y);
    if self.focused==nil then
        return
    end
    if self.focused and self.focused.onPress then
        self.focused.onPress(self.focused,x,y);
    end
end
function Frame.release(self)
    if self.focused and self.focused.onRelease then
        self.focused.onRelease(self.focused);
    end
    self.focused = nil;
end
exports.Frame = Frame
return exports
