local exports = exports or {}
local frame0 = require("src.frame.frame")
local PaintFunction = frame0.PaintFunction
local color1 = require("src.painter.color")
local Color = color1.Color
local Colors = color1.Colors
local TileSheet = TileSheet or {}
TileSheet.__index = TileSheet
function TileSheet.new(construct, ...)
    local self = setmetatable({}, TileSheet)
    if construct and TileSheet.constructor then TileSheet.constructor(self, ...) end
    return self
end
function TileSheet.constructor(self,img,column,row)
    self.img = img;
    self.column = column;
    self.row = row;
    self.img_w = img:getWidth();
    self.img_h = img:getHeight();
    self.tile_w = (self.img_w/column);
    self.tile_h = (self.img_h/row);
end
local Tile = Tile or {}
Tile.__index = Tile
function Tile.new(construct, ...)
    local self = setmetatable({}, Tile)
    if construct and Tile.constructor then Tile.constructor(self, ...) end
    return self
end
function Tile.constructor(self,x,y)
    self.x = x;
    self.y = y;
end
local function TilePainter(sheet,tile,c)
    if c==nil then c=Colors.White end
    if tile.x>=sheet.column then
        tile.x = 0;
    end
    if tile.y>=sheet.row then
        tile.y = 0;
    end
    local quad = love.graphics.newQuad(tile.x*sheet.tile_w,tile.y*sheet.tile_h,sheet.tile_w,sheet.tile_h,sheet.img_w,sheet.img_h);
    return function(x,y,width,height)
        love.graphics.setColor(c.r,c.g,c.b,1);
        love.graphics.draw(sheet.img,quad,x,y,0,width/sheet.tile_w,height/sheet.tile_h);
    end

end
local function TileSetPainter(sheet,tiles,column,spacing,c)
    if spacing==nil then spacing=0 end
    if c==nil then c=Colors.White end
    local batch = love.graphics.newSpriteBatch(sheet.img,#tiles);
    local row = math.ceil(#tiles/column);
    local tileset_w = (column*(sheet.tile_w+spacing))-spacing;
    local tileset_h = (row*(sheet.tile_h+spacing))-spacing;
    local x = 0;
    local y = 0;
    local i = 0
    while(i<#tiles) do
        do
            local tile = tiles[(i)+1];
            if tile.x>=sheet.column then
                tile.x = 0;
            end
            if tile.y>=sheet.row then
                tile.y = 0;
            end
            batch:add(love.graphics.newQuad(tile.x*sheet.tile_w,tile.y*sheet.tile_h,sheet.tile_w,sheet.tile_h,sheet.img_w,sheet.img_h),x*(sheet.tile_w+spacing),y*(sheet.tile_h+spacing));
            x = (x+1);
            if x>=column then
                x = 0;
                y = (y+1);
            end
        end
        ::__continue0::
        i = (i+1)
    end
    return function(x,y,width,height)
        love.graphics.setColor(c.r,c.g,c.b,1);
        love.graphics.draw(batch,x,y,0,width/tileset_w,height/tileset_h);
    end

end
exports.TileSheet = TileSheet
exports.Tile = Tile
exports.TilePainter = TilePainter
exports.TileSetPainter = TileSetPainter
return exports
